from langchain_core.prompts import ChatPromptTemplate
from dotenv import load_dotenv, find_dotenv
import os
from langchain_community.llms import SparkLLM
import pyperclip
import re
import sys


def dump_spark_env(id, key, secret):
    "将输入的id、key和secret加载到环境变量中。"

    os.environ['IFLYTEK_SPARK_APP_ID'] = id
    os.environ['IFLYTEK_SPARK_API_KEY'] = key
    os.environ['IFLYTEK_SPARK_API_SECRET'] = secret

def load_spark_params(root_path: str):
    "从环境变量中加载id、key和secret。"

    _ = load_dotenv(find_dotenv(f"{root_path}/.env"))    # read local .env file
    IFLYTEK_SPARK_APP_ID = os.environ["IFLYTEK_SPARK_APP_ID"] if "IFLYTEK_SPARK_APP_ID" in os.environ else None
    IFLYTEK_SPARK_API_KEY = os.environ["IFLYTEK_SPARK_API_KEY"] if "IFLYTEK_SPARK_API_KEY" in os.environ else None
    IFLYTEK_SPARK_API_SECRET = os.environ["IFLYTEK_SPARK_API_SECRET"] if "IFLYTEK_SPARK_API_SECRET" in os.environ else None
    return IFLYTEK_SPARK_APP_ID, IFLYTEK_SPARK_API_KEY, IFLYTEK_SPARK_API_SECRET

def gen_spark_params(model):
    "构造星火模型请求参数。"

    spark_url_tpl = "wss://spark-api.xf-yun.com/{}/chat"
    model_params_dict = {
        # v1.5 版本
        "v1.5": {
            "domain": "general", # 用于配置大模型版本
            "spark_url": spark_url_tpl.format("v1.1") # 云端环境的服务地址
        },
        # v2.0 版本
        "v2.0": {
            "domain": "generalv2", # 用于配置大模型版本
            "spark_url": spark_url_tpl.format("v2.1") # 云端环境的服务地址
        },
        # v3.0 版本
        "v3.0": {
            "domain": "generalv3", # 用于配置大模型版本
            "spark_url": spark_url_tpl.format("v3.1") # 云端环境的服务地址
        },
        # v3.5 版本
        "v3.5": {
            "domain": "generalv3.5", # 用于配置大模型版本
            "spark_url": spark_url_tpl.format("v3.5") # 云端环境的服务地址
        }
    }
    return model_params_dict[model]

spark_api_url = gen_spark_params(model="v3.5")["spark_url"]

def test_connection() -> bool:
    "测试api参数是否正确，可用。"

    llm = SparkLLM(spark_api_url=spark_api_url)
    result = llm.invoke("hi.")
    print(result)
    if result:
        return True
    return False


def generate_response(chain, input_text: str) -> bool:
    "生成回答。"

    result = chain.invoke({'query': input_text})
    print(f"[user: {input_text},  ai: {result}]")
    if "正确" in result:
        result = result.strip().replace('\n', '')
        result = re.findall(r"正确(.*)", result, re.S)[0]
    result = result.strip().replace('\n', '')
    print("copy: ", result, '\n')
    pyperclip.copy(result)
    if result:
        return True
    else:
        return False



def main(root_path):
    print(50*">>", '\n', "欢迎使用English Helper!!!\n", "本工具使用讯飞星火大模型v3.5.\n", "退出输入：exit\n", 50*"<<")
    correct_connection = False
    IFLYTEK_SPARK_APP_ID, IFLYTEK_SPARK_API_KEY, IFLYTEK_SPARK_API_SECRET = load_spark_params(root_path)

    while (not IFLYTEK_SPARK_APP_ID or not IFLYTEK_SPARK_API_KEY or not IFLYTEK_SPARK_API_SECRET) or not correct_connection:
        match (IFLYTEK_SPARK_APP_ID, IFLYTEK_SPARK_API_KEY, IFLYTEK_SPARK_API_SECRET, correct_connection):
            case [*params, correct_connection] if not correct_connection and all(params):
                print("测试：", params)
                dump_spark_env(*params)
                test_connection_result = test_connection()
                if test_connection_result:
                    correct_connection = True
                    print("测试成功。")
                else:
                    print("测试失败。")
            case [id, *_] if not id:
                print("请输入id：")
                IFLYTEK_SPARK_APP_ID = input()
            case [_, key, *_] if not key:
                print("请输入key：")
                IFLYTEK_SPARK_API_KEY = input()
            case [_, _, secret, *_] if not secret:
                print("请输入secret：")
                IFLYTEK_SPARK_API_SECRET = input()   

    print(50*">>", '\n', "是否自动登录？(yes or no)")
    auto_login = input()
    if auto_login.lower() == "yes":
        try:
            save_path = f"{root_path}/.env"
            print(f"保存路径: {save_path}")
            with open(save_path, 'w') as f:
                content = f'''
                    IFLYTEK_SPARK_APP_ID="{IFLYTEK_SPARK_APP_ID}"\n
                    IFLYTEK_SPARK_API_KEY="{IFLYTEK_SPARK_API_KEY}"\n
                    IFLYTEK_SPARK_API_SECRET="{IFLYTEK_SPARK_API_SECRET}"
                '''
                f.write(content)
            with open(save_path, 'r') as f:
                content = f.read()
                print(content)
            print("保存成功。请输入：")
        except:
            print("保存失败。请输入：")
    else:
        print("请输入：")

    llm = SparkLLM(spark_api_url=spark_api_url,)
    prompt = ChatPromptTemplate.from_messages([
        (
            "system",
            """
                你是一个语言辅助助手,你会对用户输入中句子进行拼写修正和改进.
                如果用户输入是个中文词语,请翻译成英文.
                如果用户输入是英文进行以下操作:
                如果句子中有错误请修正后输出.(错误包括: 拼写错误, 语法错误.)
                如果没有错误,请首先回答:‘正确’ (‘正确’之后不要包含标点符号)
                然后使用优美的词法修改句子并直接输出.
                IMPORTANT:请勿输出任何解释!输出不要用html格式。
                <input></input>标签中包含用户输入.
            """
        ),
        (
            "user",
            """
            <input>{query}</input>
            """
        )
    ])
    chain = prompt | llm

    while True:
        input_text = input()
        if input_text == "exit":
            break
        else:
            generate_response(chain, input_text)


if __name__ == "__main__":
    root_path = '/'.join([name for name in sys.argv[0].split('/')[:-1]])
    main(root_path)